﻿Public Class TrianglePoints
    Public A As PointF
    Public B As PointF
    Public C As PointF

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="sideC">The proposed initial side AB length. For ratio calculating purposes.</param>
    ''' <param name="angleA">The internal angle in Degrees at vertex A</param>
    ''' <param name="angleB">The internal angle in Degrees at vertex B</param>
    ''' <param name="angleC">The internal angle in Degrees at vertex C</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal sideC As Decimal, ByVal angleA As Decimal, ByVal angleB As Decimal, ByVal angleC As Decimal, _
                                        ByVal maxWidth As Decimal, ByVal maxHeight As Decimal, ByVal Rotation As Integer)

        'Convert Degrees to Radians
        angleA = CDec(angleA * (Math.PI / 180))
        angleB = CDec(angleB * (Math.PI / 180))
        angleC = CDec(angleC * (Math.PI / 180))

        'Calculate point B
        Dim bX As Decimal = CDec(Math.Cos(angleA) * sideC)
        Dim bY As Decimal = maxHeight - CDec(Math.Sin(angleA) * sideC)

        'Calculate relative side B length
        Dim sideB As Decimal = CDec((sideC / Math.Sin(angleC)) * Math.Sin(angleB))

        'Assign values to PointF structures
        Me.A = New PointF(0, maxHeight)
        Me.C = New PointF(sideB, maxHeight)
        Me.B = New PointF(bX, bY)

        'Array to hold PointF members
        Dim pts() As PointF = New PointF() {Me.A, Me.B, Me.C}

        'Create a Matrix and Rotate pts() Array
        Dim m As New Drawing2D.Matrix
        m.RotateAt(Rotation, New PointF(0, maxHeight))
        m.TransformPoints(pts)

        'Calculate maximum scale factor
        Dim actualWidth As Decimal = CDec(pts.Max(Function(p) p.X) - pts.Min(Function(p) p.X))
        Dim actualHeight As Decimal = CDec(pts.Max(Function(p) p.Y) - pts.Min(Function(p) p.Y))

        Dim scaleFactor As Decimal = Math.Min(maxWidth / actualWidth, maxHeight / actualHeight)

        'Scale pts() Array
        m = New Drawing2D.Matrix
        m.Scale(scaleFactor, scaleFactor)
        m.TransformPoints(pts)

        'Calculate offsets
        actualHeight = CDec(pts.Max(Function(p) p.Y) - pts.Min(Function(p) p.Y))

        Dim ox As Single = -pts.Min(Function(p) p.X)
        Dim oy As Single = -(pts.Min(Function(p) p.Y) - (maxHeight - actualHeight))

        'Offset pts() Array
        m = New Drawing2D.Matrix
        m.Translate(ox, oy)
        m.TransformPoints(pts)

        'Assign transformed Array values back to PointF member variables
        Me.A = pts(0)
        Me.B = pts(1)
        Me.C = pts(2)
    End Sub

End Class
